package com.putnam.jboss.issues;

import java.util.Hashtable;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;

/**
 * Test driver to recreate an issue where HQ non-durable, non-transacted, topic receiver hangs if the connection TTL is 
 * low (say connection-ttl=5000) and you do not publish a message to the receiver for approx 2*connection-ttl seconds
 * after the receiver has been started up. 
 * 
 * To recreate:
 * - set connection-ttl = 5000, restart HQ server
 * - start this listener
 * - start publisher after about 10 seconds
 * Expected Behavior: Receiver receives no messages.
 * 
 * @author aengineer
 *
 */
public class TopicReceiveHangsWithLowConnTTL {
    /*
     * HornetQ
     */
    private final static String JNDI_FACTORY ="org.jnp.interfaces.NamingContextFactory";
    private final static String PROVIDER_URL = "jnp://richmond2.putnaminv.com:6146";
    private final static String JNDI_PRINCIPAL = "";
    private final static String JNDI_PASSWORD = "";
    private final static String JNDI_URL_PKGS = "org.jboss.naming:org.jnp.interfaces";
    private final static String CF_LOOKUP_NAME = "/ConnectionFactory";
    private final static String DEST_LOOKUP_NAME = "/topic/ExampleTopic";
    private final static String BROKER_USER = "testuser";
    private final static String BROKER_PASSWORD = "testuser";
    private final static boolean IS_SESSION_TRANSACTED = false;
    
    public static Hashtable<String, String> JNDIEnv = new Hashtable<String, String>();
    static {
        JNDIEnv.put(Context.INITIAL_CONTEXT_FACTORY, JNDI_FACTORY);
        JNDIEnv.put(Context.PROVIDER_URL, PROVIDER_URL);
        JNDIEnv.put(Context.SECURITY_PRINCIPAL, JNDI_PRINCIPAL);
        JNDIEnv.put(Context.SECURITY_CREDENTIALS, JNDI_PASSWORD); 
        JNDIEnv.put(Context.URL_PKG_PREFIXES, JNDI_URL_PKGS);
    }
    
    /**
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        TopicReceiveHangsWithLowConnTTL client = new TopicReceiveHangsWithLowConnTTL();
        client.run(args);
    }
    
    public void run(String[] args) {
        ConnectionFactory connFactory = null;
        Connection conn = null;
        Session session = null;
        MessageProducer msgProducer = null;
        MessageConsumer msgConsumer = null;
        Destination aDestination = null;
        Message msg = null;;
        TextMessage tm = null;
        
        try {
            InitialContext ctx = new InitialContext(JNDIEnv);            
            connFactory = (ConnectionFactory) ctx.lookup(CF_LOOKUP_NAME);
            conn = connFactory.createConnection(BROKER_USER, BROKER_PASSWORD);
            
            /* Create a transacted session */
            session = conn.createSession(IS_SESSION_TRANSACTED, Session.CLIENT_ACKNOWLEDGE);
            
            aDestination = (Destination) ctx.lookup(DEST_LOOKUP_NAME);
            msgProducer = session.createProducer(aDestination);
            System.out.println("Using destination: [" + DEST_LOOKUP_NAME + "]");
            conn.start();
            

            /* Receive all messages */
            int count = 0;
            msgConsumer = session.createConsumer(aDestination);
            while (true) {
                tm = (TextMessage) msgConsumer.receive(0);
                if (tm == null) {
                    System.err.println ("No message received");
                    break;
                } else {
                    ++count;
                    if (IS_SESSION_TRANSACTED) {
                        session.commit();
                        System.out.println ("Received and committed message with id [" + tm.getJMSMessageID() + "]. Message body is [" + tm.getText() + "]");
                    } else {
                        tm.acknowledge();
                        System.out.println ("Received and acked message with id [" + tm.getJMSMessageID() + "]. Message body is [" + tm.getText() + "]");
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("An exception occurred");
            return;
        } finally {
            try {
                if (msgProducer != null) {
                    msgProducer.close();
                    msgProducer = null;
                    System.out.println("Closed producer...");
                }                
                if (msgConsumer != null) {
                    msgConsumer.close();
                    msgConsumer = null;
                    System.out.println("Closed consumer...");
                }
                if (session != null) {
                    session.close();
                    session = null;
                    System.out.println("Closed session...");
                }
                if (conn != null) {
                    conn.close();
                    conn = null;
                    System.out.println("Closed connection...");
                }
            } catch (JMSException jmse2) {
                jmse2.printStackTrace();
                System.err.println("An exception occurred when closing objects: " + jmse2.getMessage());
            }
        }
    }
}
